#!/bin/bash
#

# build_pyfits.sh 

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e4|e5|e6|e7> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    patchdir=${product_dir}/${package}/${pkgver}/patch
    sourcedir=${product_dir}/${package}/${pkgver}/source
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}
}

select_gcc()
{
    if [ "${basequal}" = "e2" ] ||  [ "${basequal}" = "gcc47" ]
    then
       gccdotver=4.7
       gccver=v4_7_1
    elif [ "${basequal}" = "e4" ] ||  [ "${basequal}" = "gcc48" ]
    then
       gccdotver=4.8
       gccver=v4_8_1
    elif [ "${basequal}" = "e5" ]
    then
       gccdotver=4.8
       gccver=v4_8_2
    elif [ "${basequal}" = "e6" ]
    then
       gccdotver=4.9
       gccver=v4_9_1
    elif [ "${basequal}" = "e7" ]
    then
        gccdotver=4.9
        gccver=v4_9_2
    else
      echo "ERROR: unrecognized qualifier ${basequal}"
      usage
      exit 1
    fi
}
setup_gcc()
{
    setup gcc ${gccver} -z ${product_dir}:${PRODUCTS}
    gccshortver=`gcc --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
    gccfullver=`gcc --version | head -1 | cut -f3 -d" "`
    if [ ${gccshortver} = ${gccdotver} ]
    then
      echo " OK - gcc version is ${gccfullver}"
    else
      echo "ERROR: wrong gcc version ${gccfullver}"
      exit 1
    fi
}

declare_product()
{
    echo "Declaring ${package} ${pkgver} for ${flvr} and ${fullqual}"
    if [ ${OS1} = "Darwin" ]
    then
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -2 -m ${package}.table -q ${fullqual} -z ${product_dir}
    else 
       ups declare ${package} ${pkgver} -r ${package}/${pkgver} -4 -m ${package}.table -q ${fullqual} -z ${product_dir}
    fi
}

function die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=pyfits
origpkgver=v3_3
pkgver=${origpkgver}a
ssibuildshims_version=v0_08_06
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

select_gcc

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

setup_gcc

if [ "${extraqual}" = "opt" ]
then
    cxxflg="-O3 -g -DNDEBUG"
    fflg="${cxxflg} -ffree-line-length-none -fPIC -J${pkgdir}/include"

elif [ "${extraqual}" = "debug" ]
then
    cxxflg="-g -O0"
    fflg="${cxxflg} -ffree-line-length-none -fPIC -J${pkgdir}/include"
elif [ "${extraqual}" = "prof" ]
then
    cxxflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
    fflg="${cxxflg} -ffree-line-length-none -fPIC -J${pkgdir}/include"
fi

if [ "${basequal}" = "e2" ] || [ "${basequal}" = "e4" ] || [ "${basequal}" = "e5" ]
then
  cxxflg="${cxxflg} -std=c++11 -Wno-deprecated-declarations"
elif [ "${basequal}" = "e6" ] || [ "${basequal}" = "e7" ]
then
	cxxflg="${cxxflg} -std=c++1y -Wno-deprecated-declarations"
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"

mkdir -p ${blddir} 
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
fi

set -x

# work in a build directory, which will not be part of any distribution
cd ${blddir}
tar xf ${tardir}/${pkgtarname}.tar.gz || die "Failed to untar ${pkgtarname}.tar.gz"
cd ${package}-${pkgdotver} || die "Failed to cd into ${package}-${pkgdotver}"
python setup.py install --prefix ${pkgdir} --single-version-externally-managed --root "/"  || die "python setup.py install failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

#replace hardwired python path 
sed '1s/^.*$/#!\/usr\/bin\/env python /' ${pkgdir}/bin/fitsdiff > ${pkgdir}/bin/fitsdifftmp
mv ${pkgdir}/bin/fitsdifftmp ${pkgdir}/bin/fitsdiff 
chmod 755 ${pkgdir}/bin/fitsdiff 

sed '1s/^.*$/#!\/usr\/bin\/env python /' ${pkgdir}/bin/fitscheck > ${pkgdir}/bin/fitschecktmp
mv ${pkgdir}/bin/fitschecktmp ${pkgdir}/bin/fitscheck 
chmod 755 ${pkgdir}/bin/fitscheck 

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "pyfits is installed at ${PYFITS_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
